function [sig,fX] = kma(X,Y,method,kernel)
%   kma  : kernel approximation method for hyperparameters optimization
%        For RBF kernel
%
%  Usage: [sig,C] = kma(X,Y,method,[sphere],[opt])
%
%    X: is an d times n matrix containing n training points in d dimensions
%    Y: is a n dimensional vector containing either the class labels
%
%    method is either:
%       'align' : optimize kernel alignment
%       'frobe' : optimize Lf norm
%        'corr' : optimize the correlation = centered alignment
%       'frobe' : optimize centered Lf norm
%    kernel is either:
%       'rbf_sphere'    : k(x,z) = exp(-0.5*||x-z||^2 / sigma^2)
%       'rbf_ellipse'   : k(x,z) = exp(-0.5*sum_i^d (x_i-z_i)^2/sigma_i^2)
%       'rbf_chol'      : k(x,z) = exp(-0.5(x-z)^tQ(x-z))
%
%    sig: the sigma(s) found by the model selection.
%    fX : is the objective function value along the optimization
%

% Initialization
[d,n] = size(X);
if (size(Y,1)~=n) | (size(Y,2)~=1)
    error('Dimension error');
end;

% Default values of the hyperparameters
switch kernel
    case 'rbf_sphere'
        sig = -log(sum(var(X,0,2)));
    case 'rbf_ellipse'
        sig = -log(var(X,0,2));
    case 'rbf_chol'
        sig=zeros(d*(d+1)/2,1);
        temp = eye(d)*sum(var(X,0,2));
        t=1;
        for i=1:d
            sig(t)=-log(temp(i,i));
            t=t+1;
            for j=i+1:d
                sig(t)=temp(i,j);
                t=t+1;
            end
        end
end

% Do the optimization of the hyperparameters
ni = 500; % Number of function evaluation of the objective function in the optimization
switch kernel
    case 'rbf_sphere';
        [sig,fX] = minimize(sig,@obj_fun_rbf_sphere,ni,X,Y,method);
        
    case 'rbf_ellipse'
        [sig,fX] = minimize(sig,@obj_fun_rbf_ellipse,ni,X,Y,method);
        
    case 'rbf_chol'
        [sig,fX] = minimize(sig,@obj_fun_rbf_chol,ni,X,Y,method);
        
end

